@@ -276,7 +276,18 @@ Association
     if(data.pdu == NULL)
     {
         // We have rejected the request
-        //return false;
+        if(!data.reject)
+        {
+            throw (*data.reject);
+        }
+        else
+        {
+            throw AssociationRejected(
+                Association::RejectedTransient,
+                Association::ULServiceProvderPresentationRelatedFunction,
+                Association::NoReasonGiven,
+                "No reject information");
+        }
     }
     else
     {
@@ -446,6 +457,11 @@ Association
 ::send_message(
     message::Message const & message, std::string const & abstract_syntax)
 {
+    if(!this->is_associated())
+    {
+        throw Exception("Not associated");
+    }
+    
     auto const transfer_syntax_it =
         this->_transfer_syntaxes_by_abstract_syntax.find(abstract_syntax);
     if(transfer_syntax_it == this->_transfer_syntaxes_by_abstract_syntax.end())
