@@ -66,10 +66,26 @@ MoveSCP
 {
     message::CMoveRequest const request(message);
 
-    auto move_association = this->_generator->get_association(request);
+    Association move_association;
+    try
+    {
+        move_association = this->_generator->get_association(request);
+    }
+    catch(odil::Exception const &)
+    {
+        message::CMoveResponse response(
+            request.get_message_id(),
+            message::CMoveResponse::RefusedMoveDestinationUnknown);
+        this->_association.send_message(
+            response, request.get_affected_sop_class_uid());
+        return;
+    }
+
     move_association.associate();
     StoreSCU store_scu(move_association);
 
+    Value::Integer final_status = message::CMoveResponse::Success;
+    DataSet status_fields;
     unsigned int remaining_sub_operations = 0;
     unsigned int completed_sub_operations=0;
     unsigned int failed_sub_operations=0;
@@ -116,33 +132,23 @@ MoveSCP
             this->_generator->next();
         }
     }
-    catch(Exception const &)
+    catch(SCP::Exception const & e)
     {
-        message::CMoveResponse response(
-            request.get_message_id(), message::CMoveResponse::UnableToProcess);
-        response.set_number_of_remaining_sub_operations(
-            remaining_sub_operations);
-        response.set_number_of_completed_sub_operations(
-            completed_sub_operations);
-        response.set_number_of_failed_sub_operations(
-            failed_sub_operations);
-        response.set_number_of_warning_sub_operations(
-            warning_sub_operations);
-        this->_association.send_message(
-            response, request.get_affected_sop_class_uid());
-        return;
+        final_status = e.status;
+        status_fields = e.status_fields;
+    }
+    catch(odil::Exception const & e)
+    {
+        status_fields.add(registry::ErrorComment, {e.what()});
+        final_status = message::CMoveResponse::UnableToProcess;
     }
 
-    message::CMoveResponse response(
-        request.get_message_id(), message::CMoveResponse::Success);
-    response.set_number_of_remaining_sub_operations(
-        remaining_sub_operations);
-    response.set_number_of_completed_sub_operations(
-        completed_sub_operations);
-    response.set_number_of_failed_sub_operations(
-        failed_sub_operations);
-    response.set_number_of_warning_sub_operations(
-        warning_sub_operations);
+    message::CMoveResponse response(request.get_message_id(), final_status);
+    response.set_status_fields(status_fields);
+    response.set_number_of_remaining_sub_operations(remaining_sub_operations);
+    response.set_number_of_completed_sub_operations(completed_sub_operations);
+    response.set_number_of_failed_sub_operations(failed_sub_operations);
+    response.set_number_of_warning_sub_operations(warning_sub_operations);
     this->_association.send_message(
         response, request.get_affected_sop_class_uid());
 }
