@@ -26,37 +26,6 @@
 #include "odil/VR.h"
 #include "odil/VRFinder.h"
 
-#define odil_read_binary(type, value, stream, byte_ordering, size) \
-type value; \
-{ \
-    uint##size##_t raw; \
-    stream.read(reinterpret_cast<char*>(&raw), sizeof(raw)); \
-    if(!stream) \
-    { \
-        throw Exception("Could not read from stream"); \
-    } \
-    if(byte_ordering == ByteOrdering::LittleEndian) \
-    { \
-        raw = little_endian_to_host(raw); \
-    } \
-    else if(byte_ordering == ByteOrdering::BigEndian) \
-    { \
-        raw = big_endian_to_host(raw); \
-    } \
-    else \
-    { \
-        throw Exception("Unknown endianness"); \
-    } \
-    value = *reinterpret_cast<type*>(&raw);\
-}
-
-#define odil_ignore(stream, size) \
-    stream.ignore(size); \
-    if(!stream) \
-    { \
-        throw Exception("Could not read from stream"); \
-    }
-
 std::string read_string(std::istream & stream, unsigned int size)
 {
     if(size == 0)
@@ -75,6 +44,66 @@ std::string read_string(std::istream & stream, unsigned int size)
 namespace odil
 {
 
+Value::Binary
+Reader
+::read_encapsulated_pixel_data(
+    std::istream & stream, ByteOrdering byte_ordering,
+    std::string transfer_syntax, bool keep_group_length)
+{
+    Value::Binary value;
+
+    // PS 3.5, A.4
+    Reader const sequence_reader(stream, transfer_syntax, keep_group_length);
+
+    bool done = false;
+    while(!done)
+    {
+        auto const tag = sequence_reader.read_tag();
+        auto const item_length = Reader::read_binary<uint32_t>(
+            stream, byte_ordering);
+
+        if(tag == registry::Item)
+        {
+            Value::Binary::value_type item_data(item_length);
+
+            if(item_length > 0)
+            {
+                stream.read(
+                    reinterpret_cast<char*>(&item_data[0]), item_length);
+                if(!stream)
+                {
+                    throw Exception("Could not read from stream");
+                }
+            }
+
+            value.push_back(item_data);
+        }
+        else if(tag == registry::SequenceDelimitationItem)
+        {
+            // No value for Sequence Delimitation Item
+            done = true;
+        }
+        else
+        {
+            throw Exception(
+                "Expected SequenceDelimitationItem, got: "+std::string(tag));
+        }
+    }
+
+    return value;
+}
+
+void
+Reader
+::ignore(std::istream & stream, std::streamsize size)
+{
+    stream.ignore(size);
+    if(!stream)
+    {
+        throw Exception("Could not read from stream");
+    }
+}
+
 Reader
 ::Reader(
     std::istream & stream, std::string const & transfer_syntax,
@@ -126,11 +155,46 @@ Tag
 Reader
 ::read_tag() const
 {
-    odil_read_binary(uint16_t, group, this->stream, this->byte_ordering, 16);
-    odil_read_binary(uint16_t, element, this->stream, this->byte_ordering, 16);
+    auto const group = this->read_binary<uint16_t>(
+        this->stream, this->byte_ordering);
+    auto const element = this->read_binary<uint16_t>(
+        this->stream, this->byte_ordering);
     return Tag(group, element);
 }
 
+uint32_t
+Reader
+::read_length(VR vr) const
+{
+    uint32_t length;
+    if(this->explicit_vr)
+    {
+        // PS 3.5, 7.1.2
+        if(is_binary(vr)
+            || vr == VR::SQ || vr == VR::UC || vr == VR::UR || vr == VR::UT)
+        {
+            Reader::ignore(this->stream, 2);
+            auto const vl = Reader::read_binary<uint32_t>(
+                this->stream, this->byte_ordering);
+            length = vl;
+        }
+        else
+        {
+            auto const vl = Reader::read_binary<uint16_t>(
+                this->stream, this->byte_ordering);
+            length = vl;
+        }
+    }
+    else
+    {
+        auto const vl = Reader::read_binary<uint32_t>(
+            this->stream, this->byte_ordering);
+        length = vl;
+    }
+
+    return length;
+}
+
 Element
 Reader
 ::read_element(Tag const & tag, DataSet const & data_set) const
@@ -148,7 +212,12 @@ Reader
 
     Value value;
 
-    if(vr == VR::IS || vr == VR::SL || vr == VR::SS ||
+    auto const vl = this->read_length(vr);
+    if(vl == 0)
+    {
+        value = Value();
+    }
+    else if(vr == VR::IS || vr == VR::SL || vr == VR::SS ||
         vr == VR::UL || vr == VR::US)
     {
         value = Value(Value::Integers());
@@ -169,7 +238,7 @@ Reader
     {
         value = Value(Value::DataSets());
     }
-    else if(vr == VR::OB || vr == VR::OF || vr == VR::OW || vr == VR::UN)
+    else if(is_binary(vr))
     {
         value = Value(Value::Binary());
     }
@@ -178,10 +247,17 @@ Reader
         throw Exception("Cannot create value for VR " + as_string(vr));
     }
 
-    Visitor visitor(
-        this->stream, vr, this->transfer_syntax, this->byte_ordering,
-        this->explicit_vr, this->keep_group_length);
-    apply_visitor(visitor, value);
+    if(!value.empty())
+    {
+        Visitor visitor(
+            this->stream, vr, vl, this->transfer_syntax, this->byte_ordering,
+            this->explicit_vr, this->keep_group_length);
+        apply_visitor(visitor, value);
+        if(vr == VR::SQ && value.as_data_sets().empty())
+        {
+            value = Value();
+        }
+    }
 
     return Element(value, vr);
 }
@@ -235,9 +311,10 @@ Reader
 
 Reader::Visitor
 ::Visitor(
-    std::istream & stream, VR vr, std::string const & transfer_syntax,
-    ByteOrdering byte_ordering, bool explicit_vr, bool keep_group_length)
-: stream(stream), vr(vr), transfer_syntax(transfer_syntax),
+    std::istream & stream, VR vr, uint32_t vl,
+    std::string const & transfer_syntax, ByteOrdering byte_ordering,
+    bool explicit_vr, bool keep_group_length)
+: stream(stream), vr(vr), vl(vl), transfer_syntax(transfer_syntax),
     byte_ordering(byte_ordering), explicit_vr(explicit_vr),
     keep_group_length(keep_group_length)
 {
@@ -248,11 +325,9 @@ Reader::Visitor::result_type
 Reader::Visitor
 ::operator()(Value::Integers & value) const
 {
-    auto const vl = this->read_length();
-
     if(this->vr == VR::IS)
     {
-        auto const string = read_string(this->stream, vl);
+        auto const string = read_string(this->stream, this->vl);
         if(!string.empty())
         {
             auto const strings = this->split_strings(string);
@@ -268,11 +343,11 @@ Reader::Visitor
         uint32_t items = 0;
         if(this->vr == VR::SL || this->vr == VR::UL)
         {
-            items = vl/4;
+            items = this->vl/4;
         }
         else if(this->vr == VR::AT || this->vr == VR::SS || this->vr == VR::US)
         {
-            items = vl/2;
+            items = this->vl/2;
         }
         else
         {
@@ -284,26 +359,26 @@ Reader::Visitor
         {
             if(this->vr == VR::SL)
             {
-                odil_read_binary(
-                    int32_t, item, this->stream, this->byte_ordering, 32);
+                auto const item = Reader::read_binary<int32_t>(
+                    this->stream, this->byte_ordering);
                 value[i] = item;
             }
             else if(this->vr == VR::SS)
             {
-                odil_read_binary(
-                    int16_t, item, this->stream, this->byte_ordering, 16);
+                auto const item = Reader::read_binary<int16_t>(
+                    this->stream, this->byte_ordering);
                 value[i] = item;
             }
             else if(this->vr == VR::UL)
             {
-                odil_read_binary(
-                    uint32_t, item, this->stream, this->byte_ordering, 32);
+                auto const item = Reader::read_binary<uint32_t>(
+                    this->stream, this->byte_ordering);
                 value[i] = item;
             }
             else if(this->vr == VR::AT || this->vr == VR::US)
             {
-                odil_read_binary(
-                    uint16_t, item, this->stream, this->byte_ordering, 16);
+                auto const item = Reader::read_binary<uint16_t>(
+                    this->stream, this->byte_ordering);
                 value[i] = item;
             }
         }
@@ -314,11 +389,9 @@ Reader::Visitor::result_type
 Reader::Visitor
 ::operator()(Value::Reals & value) const
 {
-    auto const vl = this->read_length();
-
     if(this->vr == VR::DS)
     {
-        auto const string = read_string(this->stream, vl);
+        auto const string = read_string(this->stream, this->vl);
         if(!string.empty())
         {
             auto const strings = this->split_strings(string);
@@ -337,11 +410,11 @@ Reader::Visitor
         uint32_t items = 0;
         if(this->vr == VR::FD)
         {
-            items = vl/8;
+            items = this->vl/8;
         }
         else if(this->vr == VR::FL)
         {
-            items = vl/4;
+            items = this->vl/4;
         }
         else
         {
@@ -353,14 +426,14 @@ Reader::Visitor
         {
             if(this->vr == VR::FD)
             {
-                odil_read_binary(
-                    double, item, this->stream, this->byte_ordering, 64);
+                auto const item = Reader::read_binary<double>(
+                    this->stream, this->byte_ordering);
                 value[i] = item;
             }
             else if(this->vr == VR::FL)
             {
-                odil_read_binary(
-                    float, item, this->stream, this->byte_ordering, 32);
+                auto const item = Reader::read_binary<float>(
+                    this->stream, this->byte_ordering);
                 value[i] = item;
             }
         }
@@ -388,8 +461,7 @@ Reader::Visitor
     }
     else
     {
-        auto const vl = this->read_length();
-        auto const string = read_string(this->stream, vl);
+        auto const string = read_string(this->stream, this->vl);
         if(this->vr == VR::LT || this->vr == VR::ST || this->vr == VR::UT)
         {
             value = { string };
@@ -417,12 +489,10 @@ Reader::Visitor::result_type
 Reader::Visitor
 ::operator()(Value::DataSets & value) const
 {
-    auto const vl = this->read_length();
-
-    if(vl != 0xffffffff)
+    if(this->vl != 0xffffffff)
     {
         // Explicit length sequence
-        std::string const data = read_string(this->stream, vl);
+        std::string const data = read_string(this->stream, this->vl);
         std::istringstream sequence_stream(data);
         Reader const sequence_reader(
             sequence_stream, this->transfer_syntax, this->keep_group_length);
@@ -460,7 +530,7 @@ Reader::Visitor
             else if(tag == registry::SequenceDelimitationItem)
             {
                 done = true;
-                odil_ignore(this->stream, 4);
+                Reader::ignore(this->stream, 4);
             }
             else
             {
@@ -474,51 +544,82 @@ Reader::Visitor::result_type
 Reader::Visitor
 ::operator()(Value::Binary & value) const
 {
-    auto const vl = this->read_length();
-    if(vl == 0)
+    if(this->vl == 0)
     {
         return;
     }
-    else if(vl == 0xffffffff)
+    else if(this->vl == 0xffffffff)
     {
-        value = this->read_encapsulated_pixel_data(this->stream);
+        value = Reader::read_encapsulated_pixel_data(
+            this->stream, this->byte_ordering, this->transfer_syntax,
+            this->keep_group_length);
     }
     else
     {
         value.resize(1);
         if(this->vr == VR::OB || this->vr == VR::UN)
         {
-            value[0].resize(vl);
+            value[0].resize(this->vl);
             this->stream.read(
                 reinterpret_cast<char*>(&value[0][0]), value[0].size());
         }
+        else if(this->vr == VR::OD)
+        {
+            if(this->vl%8 != 0)
+            {
+                throw Exception("Cannot read OD for odd-sized array");
+            }
+
+            value[0].resize(this->vl);
+            for(unsigned int i=0; i<value[0].size(); i+=8)
+            {
+                auto const item = Reader::read_binary<double>(
+                    this->stream, this->byte_ordering);
+                *reinterpret_cast<double*>(&value[0][i]) = item;
+            }
+        }
         else if(this->vr == VR::OF)
         {
-            if(vl%4 != 0)
+            if(this->vl%4 != 0)
             {
                 throw Exception("Cannot read OF for odd-sized array");
             }
 
-            value[0].resize(vl);
+            value[0].resize(this->vl);
             for(unsigned int i=0; i<value[0].size(); i+=4)
             {
-                odil_read_binary(
-                    float, item, this->stream, this->byte_ordering, 32);
+                auto const item = Reader::read_binary<float>(
+                    this->stream, this->byte_ordering);
                 *reinterpret_cast<float*>(&value[0][i]) = item;
             }
         }
+        else if(this->vr == VR::OL)
+        {
+            if(this->vl%4 != 0)
+            {
+                throw Exception("Cannot read OL for odd-sized array");
+            }
+
+            value[0].resize(this->vl);
+            for(unsigned int i=0; i<value[0].size(); i+=4)
+            {
+                auto const item = Reader::read_binary<uint32_t>(
+                    this->stream, this->byte_ordering);
+                *reinterpret_cast<uint32_t*>(&value[0][i]) = item;
+            }
+        }
         else if(this->vr == VR::OW)
         {
-            if(vl%2 != 0)
+            if(this->vl%2 != 0)
             {
                 throw Exception("Cannot read OW for odd-sized array");
             }
 
-            value[0].resize(vl);
+            value[0].resize(this->vl);
             for(unsigned int i=0; i<value[0].size(); i+=2)
             {
-                odil_read_binary(
-                    uint16_t, item, this->stream, this->byte_ordering, 16);
+                auto const item = Reader::read_binary<uint16_t>(
+                    this->stream, this->byte_ordering);
                 *reinterpret_cast<uint16_t*>(&value[0][i]) = item;
             }
         }
@@ -529,38 +630,6 @@ Reader::Visitor
     }
 }
 
-uint32_t
-Reader::Visitor
-::read_length() const
-{
-    uint32_t length;
-    if(this->explicit_vr)
-    {
-        if(vr == VR::OB || vr == VR::OW || vr == VR::OF || vr == VR::SQ ||
-           vr == VR::UC || vr == VR::UR || vr == VR::UT || vr == VR::UN)
-        {
-            odil_ignore(this->stream, 2);
-            odil_read_binary(
-                uint32_t, vl, this->stream, this->byte_ordering, 32);
-            length = vl;
-        }
-        else
-        {
-            odil_read_binary(
-                uint16_t, vl, this->stream, this->byte_ordering, 16);
-            length = vl;
-        }
-    }
-    else
-    {
-        odil_read_binary(
-            uint32_t, vl, this->stream, this->byte_ordering, 32);
-        length = vl;
-    }
-
-    return length;
-}
-
 Value::Strings
 Reader::Visitor
 ::split_strings(std::string const & string) const
@@ -590,8 +659,8 @@ DataSet
 Reader::Visitor
 ::read_item(std::istream & specific_stream) const
 {
-    odil_read_binary(
-        uint32_t, item_length, specific_stream, this->byte_ordering, 32);
+    auto const item_length = Reader::read_binary<uint32_t>(
+        specific_stream, this->byte_ordering);
 
     DataSet item;
     if(item_length != 0xffffffff)
@@ -616,60 +685,10 @@ Reader::Visitor
         {
             throw Exception("Unexpected tag: "+std::string(tag));
         }
-        odil_ignore(specific_stream, 4);
+        Reader::ignore(specific_stream, 4);
     }
 
     return item;
 }
 
-Value::Binary
-Reader::Visitor
-::read_encapsulated_pixel_data(std::istream & specific_stream) const
-{
-    Value::Binary value;
-
-    // PS 3.5, A.4
-    Reader const sequence_reader(
-        specific_stream, this->transfer_syntax, this->keep_group_length);
-    bool done = false;
-    while(!done)
-    {
-        auto const tag = sequence_reader.read_tag();
-        odil_read_binary(
-            uint32_t, item_length, specific_stream, this->byte_ordering, 32);
-
-        if(tag == registry::Item)
-        {
-            Value::Binary::value_type item_data(item_length);
-
-            if(item_length > 0)
-            {
-                specific_stream.read(
-                    reinterpret_cast<char*>(&item_data[0]), item_length);
-                if(!stream)
-                {
-                    throw Exception("Could not read from stream");
-                }
-            }
-
-            value.push_back(item_data);
-        }
-        else if(tag == registry::SequenceDelimitationItem)
-        {
-            // No value for Sequence Delimitation Item
-            done = true;
-        }
-        else
-        {
-            throw Exception(
-                "Expected SequenceDelimitationItem, got: "+std::string(tag));
-        }
-    }
-
-    return value;
 }
-
-}
-
-#undef odil_ignore
-#undef odil_read_binary
