@@ -43,6 +43,21 @@ public:
     /// @brief Flag to keep or discard group length tags.
     bool keep_group_length;
 
+    /**
+     * @brief Read binary data from an stream encoded with the given endianness,
+     * ensure stream is still good.
+     */
+    template<typename T>
+    static T read_binary(std::istream & stream, ByteOrdering byte_ordering);
+
+    /// @brief Read pixel data in encapsulated form.
+    static Value::Binary read_encapsulated_pixel_data(
+        std::istream & stream, ByteOrdering byte_ordering,
+        std::string transfer_syntax, bool keep_group_length=false);
+
+    /// @brief Ignore data from a stream, ensure stream is still good.
+    static void ignore(std::istream & stream, std::streamsize size);
+
     /**
      * @brief Build a reader, derive byte ordering and explicit-ness of VR
      * from transfer syntax.
@@ -58,6 +73,9 @@ public:
     /// @brief Read a tag.
     Tag read_tag() const;
 
+    /// @brief Read the length of an element.
+    uint32_t read_length(VR vr) const;
+
     /**
      * @brief Read an element (VR and value), try to guess the VR from the tag,
      * partially read data set, and transfer syntax for implicit VR transfer
@@ -80,6 +98,7 @@ private:
 
         std::istream & stream;
         VR vr;
+        uint32_t vl;
 
         std::string transfer_syntax;
         ByteOrdering byte_ordering;
@@ -87,8 +106,9 @@ private:
         bool keep_group_length;
 
         Visitor(
-            std::istream & stream, VR vr, std::string const & transfer_syntax,
-            ByteOrdering byte_ordering, bool explicit_vr, bool keep_group_length);
+            std::istream & stream, VR vr, uint32_t vl,
+            std::string const & transfer_syntax, ByteOrdering byte_ordering,
+            bool explicit_vr, bool keep_group_length);
 
         result_type operator()(Value::Integers & value) const;
         result_type operator()(Value::Reals & value) const;
@@ -96,7 +116,7 @@ private:
         result_type operator()(Value::DataSets & value) const;
         result_type operator()(Value::Binary & value) const;
 
-        uint32_t read_length() const;
+        // uint32_t read_length() const;
 
         Value::Strings split_strings(std::string const & string) const;
         DataSet read_item(std::istream & specific_stream) const;
@@ -107,4 +127,6 @@ private:
 
 }
 
+#include "odil/Reader.txx"
+
 #endif // _aa2965aa_e891_4713_9c90_e8eacd2944ea
