@@ -31,11 +31,10 @@ public:
     /// @brief Set the affected SOP class
     void set_affected_sop_class(std::string const & sop_class);
     
-    /// @brief Perform DICOM ping
-    void echo() const;
-
 protected:
+    /// @brief Association with peer.
     Association & _association;
+    
     /// @brief Affected SOP class.
     std::string _affected_sop_class;
 };
