@@ -46,6 +46,19 @@ public:
     /// @brief Presence of group length elements.
     bool use_group_length;
 
+    /**
+     * @brief Write binary data to an stream encoded with the given endianness,
+     * ensure stream is still good.
+     */
+    template<typename T>
+    static void write_binary(
+        T const & value, std::ostream & stream, ByteOrdering byte_ordering);
+
+    /// @brief Write pixel data in encapsulated form.
+    static void write_encapsulated_pixel_data(
+        Value::Binary const & value, std::ostream & stream,
+        ByteOrdering byte_ordering, bool explicit_vr);
+
     /// @brief Build a writer.
     Writer(
         std::ostream & stream,
@@ -107,11 +120,11 @@ private:
 
         template<typename T>
         void write_strings(T const & sequence, char padding) const;
-
-        void write_encapsulated_pixel_data(Value::Binary const & value) const;
     };
 };
 
 }
 
+#include "odil/Writer.txx"
+
 #endif // _ca5c06d2_04f9_4009_9e98_5607e1060379
